# Import metrics directly from torchmetrics
# from torchmetrics.retrieval import RetrievalRecall, RetrievalPrecision

# Import custom metrics from visionmetrics
from visionmetrics.retrieval.precision_recall import (
    RetrievalMAP, RetrievalPrecision, RetrievalPrecisionRecallCurveNPoints,
    RetrievalRecall)



# Import metrics directly from torchmetrics
from torchmetrics.classification import (BinaryAUROC, MulticlassAccuracy,
                                         MulticlassAUROC,
                                         MulticlassAveragePrecision,
                                         MulticlassCalibrationError,
                                         MulticlassConfusionMatrix,
                                         MulticlassPrecision, MulticlassRecall,
                                         MultilabelAccuracy, MultilabelAUROC,
                                         MultilabelAveragePrecision,
                                         MultilabelConfusionMatrix)

# Import custom metrics from visionmetrics
from visionmetrics.classification.precision_recall import (MultilabelF1Score,
                                                           MultilabelPrecision,
                                                           MultilabelRecall)



import torch
from torchmetrics import detection
from typing import List, Dict, Tuple


class MeanAveragePrecision(detection.mean_ap.MeanAveragePrecision):
    """
    Mean Average Precision (mAP) metric for object detection task.

    This implementation extends the `torchmetrics` implementation of mAP to:

    1. Accept predictions and targets in a different format. The `update` method expects predictions and targets to be a list of lists of lists, where each inner list corresponds to a single image.
    The innermost list contains the predicted or ground truth boxes for that image, where each box is represented as a list of 6 (or 5 for target with no score) elements:
    [label, score, L, T, R, B], where L, T, R, B are the coordinates of the box in the format 'xyxy' (xmin, ymin, xmax, ymax).

    2. Accept both relative and absolute coordinates.
    NOTE: torchmetrics.*.MeanAveragePrecision only expects absolute coordinates so it is possible after some version update MeanAveragePrecision might break suddenly.

    3. Only compute the following metrics: map, map_50, map_75, map_per_class (which are independent of the coordinate format).

    Example:
    ```
    predictions = [[[0, 0.9, 10, 20, 50, 100], [1, 0.8, 30, 40, 80, 120]], [[1, 0.7, 20, 30, 60, 90]]]
    targets = [[[0, 10, 20, 50, 100], [1, 30, 40, 80, 120]], [[1, 20, 30, 60, 90]]]

    metric = MeanAveragePrecision()
    metric.update(predictions, targets)
    ap = metric.compute()
    ```
    """

    def __init__(self, box_format='xyxy', **kwargs):
        # TODO: add support for other box formats
        if box_format != 'xyxy':
            raise ValueError(f'Expected box format to be "xyxy", got {box_format}')
        super().__init__(box_format=box_format, **kwargs)

    def update(self, predictions: List[List[List[float]]], targets: List[List[List[float]]]) -> None:
        predictions, targets = self._preprocess(predictions, targets)
        super().update(predictions, targets)

    def compute(self):
        result = super().compute()
        return {k: v for k, v in result.items() if k in ['map', 'map_50', 'map_75', 'map_per_class']}

    def _preprocess(self, predictions: List[List[List[float]]], targets: List[List[List[float]]]) -> Tuple[List[Dict[str, torch.Tensor]], List[Dict[str, torch.Tensor]]]:
        predictions = [self._convert_to_dict(p) for p in predictions]
        targets = [self._convert_to_dict(t, scores=False) for t in targets]
        return predictions, targets

    @staticmethod
    def _convert_to_dict(boxes: List[List[float]], scores: bool = True) -> Dict[str, torch.Tensor]:
        """
        Args:
            boxes (list): list of boxes. Each box is a list of 6 (or 5 when no score) elements: [label, score, L, T, R, B]
            score (bool): whether to include scores in the output dictionary
        """
        if not boxes:
            boxes = torch.empty(0, 6) if scores else torch.empty(0, 5)
        else:
            boxes = torch.tensor(boxes) if not isinstance(boxes, torch.Tensor) else boxes

        if scores:
            return {'boxes': boxes[:, -4:], 'labels': boxes[:, 0].to(torch.int), 'scores': boxes[:, 1]}
        else:
            return {'boxes': boxes[:, -4:], 'labels': boxes[:, 0].to(torch.int)}



import numpy as np
import torch
from visionmetrics.matting.matting_eval_base import MattingEvaluatorBase


class L1Error(MattingEvaluatorBase):
    """
    L1 error evaluator
    """

    def __init__(self):
        super().__init__(metric='L1Err')

    def update(self, predictions, targets):
        """ Adding predictions and ground truth of images for image matting task
        Args:
            predictions: list of image matting predictions, [matting1, matting2, ...]. Shape: (N, ), type: PIL image object
            targets: list of image matting ground truth, [gt1, gt2, ...]. Shape: (N, ), type: PIL image object
        """

        assert len(predictions) == len(targets)

        self._num_samples += len(predictions)
        for pred_mask, gt_mask in zip(predictions, targets):
            pred_mask = torch.tensor(np.array(pred_mask))
            gt_mask = torch.tensor(np.array(gt_mask))
            mean_l1 = torch.abs(pred_mask.to(torch.float)-gt_mask.to(torch.float)).mean()
            self._metric_sum += mean_l1.numpy()



import numpy as np
import torch
from visionmetrics.matting.matting_eval_base import MattingEvaluatorBase


class ForegroundIOU(MattingEvaluatorBase):
    """
    Foreground intersection-over-union evaluator
    """

    def __init__(self):
        super().__init__(metric='fgIOU')

    def update(self, predictions, targets):
        """ Adding predictions and ground truth of images for image matting task
        Args:
            predictions: list of image matting predictions, [matting1, matting2, ...]. Shape: (N, ), type: PIL image object
            targets: list of image matting ground truth, [gt1, gt2, ...]. Shape: (N, ), type: PIL image object
        """

        assert len(predictions) == len(targets)

        num_class = 2
        self._num_samples += len(predictions)
        for pred_mask, gt_mask in zip(predictions, targets):
            pred_mask = torch.tensor(np.array(pred_mask))
            gt_mask = torch.tensor(np.array(gt_mask))
            pred_binmask, gt_binmask = self._preprocess(pred_mask, gt_mask)
            if torch.all(gt_binmask == 0):
                res = 1 if torch.all(pred_binmask == 0) else 0
                self.metric_sum += res
                continue

            label = num_class * gt_binmask + pred_binmask
            count = torch.bincount(label.flatten(), minlength=num_class**2)
            confusion_matrix = count.reshape(num_class, num_class)
            iou = torch.diag(confusion_matrix) / (confusion_matrix.sum(axis=1) + confusion_matrix.sum(axis=0) - torch.diag(confusion_matrix) + 1e-10)
            self._metric_sum += iou[1]



from .caption_eval_base import ImageCaptionEvaluatorBase


class BleuScore(ImageCaptionEvaluatorBase):
    """
    BLEU score evaluator for image caption task. For more details, refer to http://www.aclweb.org/anthology/P02-1040.pdf.
    """

    def __init__(self):
        super().__init__(metric='Bleu')
        self.predictions = []
        self.targets = []



import numpy as np
import torch
from visionmetrics.matting.mean_iou import MeanIOU


class BoundaryMeanIOU(MeanIOU):
    """
    Boundary mean intersection-over-union evaluator
    """

    def __init__(self):
        super().__init__(metric='b_mIOU')

    def _preprocess(self, pred_mask, gt_mask):
        pred_mask = torch.tensor(np.array(pred_mask))
        gt_mask = torch.tensor(np.array(gt_mask))
        pred_binmask = self._convert2binary(pred_mask)
        gt_binmask = self._convert2binary(gt_mask)
        gt_boundary_mask, pred_boundary_mask = self._create_contour_mask(gt_binmask, pred_binmask)

        return pred_boundary_mask.to(torch.int64), gt_boundary_mask.to(torch.int64)



from torchmetrics import Metric


class ImageCaptionEvaluatorBase(Metric):
    """
    Base class for image caption metric evaluator
    """

    def __init__(self, metric):
        super().__init__()
        self.metric = metric
        self.add_state("predictions", default=[], dist_reduce_fx="cat")
        self.add_state("targets", default=[], dist_reduce_fx="cat")

    def update(self, predictions, targets):
        """ Evaluate list of image with image caption results using pycocoimcap tools.
        Args:
            predictions: list of string predictions [caption1, caption2, ...], shape: (N, ), type: string
            targets: list of string ground truth for image caption task: [[gt1, gt2, ...], [gt1, gt2, ...], ...], type: string
        """
        self.targets += targets
        self.predictions += predictions

    def compute(self, **kwargs):
        from .coco_evalcap_utils import ImageCaptionCOCOEval, ImageCaptionCOCO, ImageCaptionWrapper
        imcap_predictions, imcap_targets = ImageCaptionWrapper.convert(self.predictions, self.targets)
        coco = ImageCaptionCOCO(imcap_targets)
        cocoRes = coco.loadRes(imcap_predictions)
        cocoEval = ImageCaptionCOCOEval(coco, cocoRes, self.metric)
        cocoEval.params['image_id'] = cocoRes.getImgIds()
        cocoEval.evaluate()
        result = cocoEval.eval
        return result



import json
import pathlib
import unittest

from visionmetrics.caption import BleuScore, CIDErScore, METEORScore, ROUGELScore


class TestImageCaptionEvaluator(unittest.TestCase):
    predictions_file = pathlib.Path(__file__).resolve().parent / 'data' / 'image_caption_prediction.json'
    ground_truth_file = pathlib.Path(__file__).resolve().parent / 'data' / 'image_caption_gt.json'
    imcap_predictions, imcap_targets = [], []
    predictions_dict = json.loads(predictions_file.read_text())
    ground_truth_dict = json.loads(ground_truth_file.read_text())

    gts_by_id = {}
    predictions_by_id = {pred['image_id']: pred['caption'] for pred in predictions_dict}

    for gt in ground_truth_dict['annotations']:
        if not gt['image_id'] in gts_by_id:
            gts_by_id[gt['image_id']] = []
        gts_by_id[gt['image_id']].append(gt['caption'])
    for key, value in predictions_by_id.items():
        imcap_predictions.append(value)
        imcap_targets.append(gts_by_id[key])

    def test_image_caption_blue_score_evaluator(self):
        evaluator = BleuScore()
        evaluator.update(predictions=self.imcap_predictions, targets=self.imcap_targets)
        report = evaluator.compute()
        self.assertAlmostEqual(report["Bleu_1"], 0.783228681385441)
        self.assertAlmostEqual(report["Bleu_2"], 0.6226378540059051)
        self.assertAlmostEqual(report["Bleu_3"], 0.47542636331846966)
        self.assertAlmostEqual(report["Bleu_4"], 0.3573567238999926)

    def test_image_caption_meteor_score_evaluator(self):
        evaluator = METEORScore()
        evaluator.update(predictions=self.imcap_predictions, targets=self.imcap_targets)
        report = evaluator.compute()
        self.assertAlmostEqual(report["METEOR"], 0.2878681068021112)

    def test_image_caption_rouge_l_score_evaluator(self):
        evaluator = ROUGELScore()
        evaluator.update(predictions=self.imcap_predictions, targets=self.imcap_targets)
        report = evaluator.compute()
        self.assertAlmostEqual(report["ROUGE_L"], 0.5774238052522583)

    def test_image_caption_cider_score_evaluator(self):
        evaluator = CIDErScore()
        evaluator.update(predictions=self.imcap_predictions, targets=self.imcap_targets)
        report = evaluator.compute()
        self.assertAlmostEqual(report["CIDEr"], 1.2346054374217474)



from typing import Any

import torch
import torchmetrics
from torchmetrics.functional.classification import binary_precision_recall_curve


class RetrievalExtend:
    def update(self, predictions, targets):
        """ torchmetrics implementation of Retrieval* metrics expects an indexes tensor corresponding to each query.
        For e.g. preds = torch.tensor([[0.6, 0.3],[0.2, 0.7])) --> indexes = torch.tensor([[0, 0], [1, 1]])
        """
        indexes = self._create_indexes(predictions)
        super().update(predictions, targets, indexes)

    def _create_indexes(self, predictions):
        if len(predictions.shape) == 1:
            indexes = torch.zeros_like(predictions)
        else:
            indexes = torch.arange(predictions.shape[0]).unsqueeze(1).repeat(1, predictions.shape[1])
        return indexes.long()


class RetrievalPrecision(RetrievalExtend, torchmetrics.retrieval.RetrievalPrecision):
    pass


class RetrievalRecall(RetrievalExtend, torchmetrics.retrieval.RetrievalRecall):
    pass


class RetrievalMAP(RetrievalExtend, torchmetrics.retrieval.RetrievalMAP):
    pass


class RetrievalPrecisionRecallCurveNPoints(torchmetrics.Metric):
    def __init__(self, n_points, **kwargs: Any) -> None:
        self.n_points = n_points
        super().__init__(**kwargs)

        self.add_state("recall_thresholds", default=torch.linspace(1, 0, self.n_points), dist_reduce_fx=None)
        self.add_state("precision_averaged", default=torch.zeros(self.n_points), dist_reduce_fx="sum")
        self.add_state("total_samples", default=torch.zeros(1), dist_reduce_fx="sum")

    def update(self, predictions, targets):
        n_samples = predictions.shape[0]
        self.total_samples += n_samples
        for i in range(n_samples):
            precision_interp = self._compute_precision_recall_interp(predictions[i], targets[i], self.recall_thresholds)
            self.precision_averaged += precision_interp

    def compute(self):
        return self.precision_averaged / self.total_samples, self.recall_thresholds

    def _compute_precision_recall_interp(self, prediction, target, recall_thresholds):
        assert len(prediction) == len(target)
        assert len(target.shape) == 1

        # NOTE: thresholds are calculated in this way to achieve parity with vision-evaluation
        # that uses sklearn-based precision_recall_curve which uses thresholds in this way
        thresholds = torch.unique(prediction)

        precision, recall, _ = binary_precision_recall_curve(prediction, target, thresholds)
        precision_interp = torch.zeros_like(recall_thresholds)
        mask = recall_thresholds.unsqueeze(-1) <= recall
        precision_interp = (precision * mask).max(dim=-1).values
        return precision_interp



import torchmetrics
from typing import Literal

from visionmetrics.prediction_filters import TopKPredictionFilter


class MultilabelExtend:
    """
    Extend torchmetrics.classification.Multilabel* metrics to support:
        1. top_k filtering
        2. 'indices' prediction_mode
    """

    def __init__(self, top_k: int = None, prediction_mode: Literal['prob', 'indices'] = 'prob'):
        self.top_k = top_k
        self.prediction_mode = prediction_mode

    def update(self, predictions, targets):
        # torchmetrics only supports prob mode
        if self.top_k is None and self.prediction_mode == 'prob':
            super().update(predictions, targets)
        else:
            self.top_k = predictions.shape[1] if self.top_k is None else self.top_k
            pred_filter = TopKPredictionFilter(k=self.top_k, prediction_mode=self.prediction_mode)
            topk_preds = pred_filter.filter(predictions)
            super().update(topk_preds, targets)


class MultilabelPrecision(MultilabelExtend, torchmetrics.classification.MultilabelPrecision):
    def __init__(self, top_k=None, prediction_mode='prob', *args, **kwargs):
        MultilabelExtend.__init__(self, top_k=top_k, prediction_mode=prediction_mode)
        torchmetrics.classification.MultilabelPrecision.__init__(self, *args, **kwargs)


class MultilabelRecall(MultilabelExtend, torchmetrics.classification.MultilabelRecall):
    def __init__(self, top_k=None, prediction_mode='prob', *args, **kwargs):
        MultilabelExtend.__init__(self, top_k=top_k, prediction_mode=prediction_mode)
        torchmetrics.classification.MultilabelRecall.__init__(self, *args, **kwargs)


class MultilabelF1Score(MultilabelExtend, torchmetrics.classification.MultilabelF1Score):
    def __init__(self, top_k=None, prediction_mode='prob', *args, **kwargs):
        MultilabelExtend.__init__(self, top_k=top_k, prediction_mode=prediction_mode)
        torchmetrics.classification.MultilabelF1Score.__init__(self, *args, **kwargs)



from visionmetrics.caption.caption_eval_base import ImageCaptionEvaluatorBase


class ROUGELScore(ImageCaptionEvaluatorBase):
    """
    ROUGE_L score evaluator for image caption task. For more details, refer to http://anthology.aclweb.org/W/W04/W04-1013.pdf
    """

    def __init__(self):
        super().__init__(metric='ROUGE_L')



import pathlib
import unittest

from PIL import Image
from visionmetrics.matting import L1Error, MeanIOU,  BoundaryMeanIOU, ForegroundIOU, BoundaryForegroundIOU


class TestImageMattingEvaluator(unittest.TestCase):

    image_matting_predictions = []
    image_matting_targets = []

    image_matting_predictions.append(Image.open(pathlib.Path(__file__).resolve().parent / 'data' / 'test_0.png'))
    image_matting_predictions.append(Image.open(pathlib.Path(__file__).resolve().parent / 'data' / 'test_1.png'))
    image_matting_targets.append(Image.open(pathlib.Path(__file__).resolve().parent / 'data' / 'gt_0.png'))
    image_matting_targets.append(Image.open(pathlib.Path(__file__).resolve().parent / 'data' / 'gt_1.png'))

    def test_image_matting_mean_iou_evaluator(self):
        evaluator = MeanIOU()
        evaluator.update(predictions=self.image_matting_predictions, targets=self.image_matting_targets)
        report = evaluator.compute()
        self.assertAlmostEqual(report["mIOU"], 0.4530012134867148)

    def test_image_matting_foreground_iou_evaluator(self):
        evaluator = ForegroundIOU()
        evaluator.update(predictions=self.image_matting_predictions, targets=self.image_matting_targets)
        report = evaluator.compute()
        self.assertAlmostEqual(report["fgIOU"], 0.23992256209190865)

    def test_image_matting_l1_error_evaluator(self):
        evaluator = L1Error()
        evaluator.update(predictions=self.image_matting_predictions, targets=self.image_matting_targets)
        report = evaluator.compute()
        self.assertAlmostEqual(report["L1Err"], 77.07374954223633)

    def test_image_matting_boundary_mean_iou_evaluator(self):
        evaluator = BoundaryMeanIOU()
        evaluator.update(predictions=self.image_matting_predictions, targets=self.image_matting_targets)
        report = evaluator.compute()
        self.assertAlmostEqual(report["b_mIOU"], 0.6022812)

    def test_image_matting_boundary_foreground_iou_evaluator(self):
        evaluator = BoundaryForegroundIOU()
        evaluator.update(predictions=self.image_matting_predictions, targets=self.image_matting_targets)
        report = evaluator.compute()
        self.assertAlmostEqual(report["b_fgIOU"], 0.2460145344436508)



import torch


class TopKPredictionFilter:

    def __init__(self, k: int, prediction_mode='prob'):
        """
        Args:
            k: k predictions with highest confidence
            prediction_mode: can be 'indices' or 'prob', indicating whether the predictions are a set of class indices or predicted probabilities.
        """
        assert k >= 0
        assert prediction_mode == 'prob' or prediction_mode == 'indices', f"Prediction mode {prediction_mode} is not supported!"

        self.prediction_mode = prediction_mode
        self.k = k

    def filter(self, predictions, return_mode='vec'):
        """ Return k class predictions with highest confidence.
        Args:
            predictions:
                when 'prediction_mode' is 'prob', refers to predicted probabilities of N samples belonging to C classes. Shape (N, C)
                when 'prediction_mode' is 'indices', refers to indices of M highest confidence of C classes in descending order, for each of the N samples. Shape (N, M)
            return_mode: can be 'indices' or 'vec', indicating whether return value is a set of class indices or 0-1 vector

        Returns:
            k labels with highest probabilities, for each sample
        """

        k = min(predictions.shape[1], self.k)

        if self.prediction_mode == 'prob':
            if k == 0:
                top_k_pred_indices = torch.tensor([[] for i in range(predictions.shape[1])], dtype=int)
            elif k == 1:
                top_k_pred_indices = torch.argmax(predictions, axis=1)
                top_k_pred_indices = top_k_pred_indices.reshape((-1, 1))
            else:
                top_k_pred_indices = torch.topk(predictions, k=k, dim=1).indices
        else:
            top_k_pred_indices = predictions[:, :k]

        if return_mode == 'indices':
            return list(top_k_pred_indices)
        else:
            preds = torch.zeros_like(predictions, dtype=int)
            row_index = torch.arange(predictions.shape[0]).repeat_interleave(k)
            col_index = top_k_pred_indices.reshape((1, -1))
            preds[row_index, col_index] = 1

            return preds



import unittest

import torch

from visionmetrics.detection import MeanAveragePrecision


class TestDetection(unittest.TestCase):
    def test_perfect_one_image_absolute_coordinates(self):

        PREDICTIONS = [[[0, 1.0, 0, 0, 10, 10],
                        [1, 1.0, 5, 5, 10, 10],
                        [2, 1.0, 1, 1, 5, 5]]]

        TARGETS = [[[0, 0, 0, 10, 10],
                    [1, 5, 5, 10, 10],
                    [2, 1, 1, 5, 5]]]

        metric = MeanAveragePrecision()
        metric.update(PREDICTIONS, TARGETS)
        result = metric.compute()
        self.assertEqual(result['map_50'], 1.0)

    def test_perfect_one_image_relative_coordinates(self):

        PREDICTIONS = [[[0, 1.0, 0, 0, 1, 1],
                        [1, 1.0, 0.5, 0.5, 1, 1],
                        [2, 1.0, 0.1, 0.1, 0.5, 0.5]]]

        TARGETS = [[[0, 0, 0, 1, 1],
                    [1, 0.5, 0.5, 1, 1],
                    [2, 0.1, 0.1, 0.5, 0.5]]]

        metric = MeanAveragePrecision()
        metric.update(PREDICTIONS, TARGETS)
        result = metric.compute()
        self.assertEqual(result['map_50'], 1.0)

    def test_wrong_one_image(self):
        PREDICTIONS = [[[0, 1.0, 0, 0, 1, 1],
                        [0, 1.0, 0.5, 0.5, 1, 1],
                        [1, 1.0, 0.5, 0.5, 1, 1]]]

        TARGETS = [[[0, 0, 0, 1, 1],
                    [1, 0.5, 0.5, 1, 1]]]

        metric = MeanAveragePrecision()
        metric.update(PREDICTIONS, TARGETS)
        result = metric.compute()
        self.assertEqual(result['map_50'], 1.0)

    def test_perfect_two_images(self):
        PREDICTIONS = [[[0, 1.0, 0, 0, 1, 1],
                        [1, 1.0, 0.5, 0.5, 1, 1]],
                       [[2, 1.0, 0.1, 0.1, 0.5, 0.5]]]

        TARGETS = [[[0, 0, 0, 1, 1],
                    [1, 0.5, 0.5, 1, 1]],
                   [[2, 0.1, 0.1, 0.5, 0.5]]]

        metric = MeanAveragePrecision()
        metric.update(PREDICTIONS, TARGETS)
        result = metric.compute()
        self.assertEqual(result['map_50'], 1.0)

    def test_two_batches(self):

        PREDICTIONS = [[[0, 1.0, 0, 0, 1, 1],
                        [1, 1.0, 0.5, 0.5, 1, 1]],
                       [[2, 1.0, 0.1, 0.1, 0.5, 0.5]]]

        TARGETS = [[[0, 0, 0, 1, 1],
                    [1, 0.5, 0.5, 1, 1]],
                   [[2, 0.1, 0.1, 0.5, 0.5]]]

        metric = MeanAveragePrecision()
        metric.update(PREDICTIONS, TARGETS)

        PREDICTIONS = [[[0, 1.0, 0.9, 0.9, 1, 1],  # Wrong
                        [1, 1.0, 0.5, 0.5, 1, 1]],
                       [[2, 1.0, 0.1, 0.1, 0.5, 0.5]]]

        TARGETS = [[[0, 0, 0, 1, 1],
                    [1, 0.5, 0.5, 1, 1]],
                   [[2, 0.1, 0.1, 0.5, 0.5]]]

        metric.update(PREDICTIONS, TARGETS)

        result = metric.compute()
        self.assertAlmostEqual(result['map_50'].item(), 0.834983, places=5)

    def test_iou_thresholds(self):
        metric = MeanAveragePrecision(iou_thresholds=[0.5])
        PREDICTIONS = [[[0, 1.0, 0.5, 0.5, 1, 1],
                        [1, 1.0, 0.5, 0.5, 1, 1]]]

        TARGETS = [[[0, 0, 0, 1, 1],
                    [1, 0.5, 0.5, 1, 1]]]

        metric.update(PREDICTIONS, TARGETS)
        result = metric.compute()
        self.assertAlmostEqual(result['map_50'].item(), 0.5, places=5)

        metric = MeanAveragePrecision(iou_thresholds=[0.2])
        PREDICTIONS = [[[0, 1.0, 0.5, 0.5, 1, 1],
                        [1, 1.0, 0.5, 0.5, 1, 1]]]

        TARGETS = [[[0, 0, 0, 1, 1],
                    [1, 0.5, 0.5, 1, 1]]]

        metric.update(PREDICTIONS, TARGETS)
        result = metric.compute()
        self.assertAlmostEqual(result['map'].item(), 1.0, places=5)

    def test_no_predictions(self):
        PREDICTIONS = [[]]
        TARGETS = [[[0, 0, 0, 1, 1],
                    [1, 0.5, 0.5, 1, 1],
                    [2, 0.1, 0.1, 0.5, 0.5]]]

        metric = MeanAveragePrecision()
        metric.update(PREDICTIONS, TARGETS)
        result = metric.compute()
        self.assertAlmostEqual(result['map_50'].item(), 0.0, places=5)

    def test_no_targets(self):
        PREDICTIONS = [[[0, 1.0, 0, 0, 1, 1],
                        [1, 1.0, 0.5, 0.5, 1, 1],
                        [2, 1.0, 0.1, 0.1, 0.5, 0.5]]]
        TARGETS = [[]]

        metric = MeanAveragePrecision(iou_thresholds=[0.5])
        metric.update(PREDICTIONS, TARGETS)
        result = metric.compute()
        self.assertAlmostEqual(result['map_50'].item(), -1, places=5)

    def test_cat_id_remap(self):
        PREDICTIONS = [[(0, 1.0, 0, 0, 1, 1),
                        [1, 1.0, 0.5, 0.5, 1, 1],
                        [2, 1.0, 0.1, 0.1, 0.5, 0.5]]]

        TARGETS = [[[0, 0, 0, 1, 1],
                    [0, 0.5, 0.5, 1, 1],
                    [2, 0.1, 0.1, 0.5, 0.5]]]

        metric = MeanAveragePrecision(iou_thresholds=[0.5])
        metric.update(PREDICTIONS, TARGETS)
        result = metric.compute()

        PREDICTIONS_REMAP_CAT_ID = [[[0, 1.0, 0, 0, 1, 1],
                                     [2, 1.0, 0.5, 0.5, 1, 1],
                                     [1, 1.0, 0.1, 0.1, 0.5, 0.5]]]

        TARGETS_REMAP_CAT_ID = [[[0, 0, 0, 1, 1],
                                 [0, 0.5, 0.5, 1, 1],
                                 [1, 0.1, 0.1, 0.5, 0.5]]]

        metric = MeanAveragePrecision(iou_thresholds=[0.5])
        metric.update(PREDICTIONS_REMAP_CAT_ID, TARGETS_REMAP_CAT_ID)
        result_remap_cat_id = metric.compute()

        for k in result.keys():
            self.assertTrue(torch.allclose(result[k], result_remap_cat_id[k]))


if __name__ == '__main__':
    unittest.main()



import unittest

import torch

from visionmetrics.classification import (BinaryAUROC, MulticlassAccuracy,
                                          MulticlassAUROC,
                                          MulticlassAveragePrecision,
                                          MulticlassCalibrationError,
                                          MulticlassPrecision,
                                          MultilabelAccuracy, MultilabelAUROC,
                                          MultilabelAveragePrecision,
                                          MultilabelF1Score,
                                          MultilabelPrecision,
                                          MultilabelRecall)


class TestMulticlassClassification(unittest.TestCase):
    TARGETS = [
        torch.tensor([1, 0, 0, 0, 1, 1, 0, 0, 0, 1]),
        torch.tensor([1, 0, 2, 0, 1, 2, 0, 0, 0, 1, 2, 1, 2, 2, 0])]
    PREDICTIONS = [
        torch.tensor([[1, 0],
                      [0, 1],
                      [0.5, 0.5],
                      [0.1, 0.9],
                      [0.44, 0.56],
                      [0.09, 0.91],
                      [0.91, 0.09],
                      [0.37, 0.63],
                      [0.34, 0.66],
                      [0.89, 0.11]]),
        torch.tensor([[0.99, 0.01, 0],
                      [0, 0.99, 0.01],
                      [0.51, 0.49, 0.0],
                      [0.09, 0.8, 0.11],
                      [0.34, 0.36, 0.3],
                      [0.09, 0.90, 0.01],
                      [0.91, 0.06, 0.03],
                      [0.37, 0.60, 0.03],
                      [0.34, 0.46, 0.2],
                      [0.79, 0.11, 0.1],
                      [0.34, 0.16, 0.5],
                      [0.04, 0.56, 0.4],
                      [0.04, 0.36, 0.6],
                      [0.04, 0.36, 0.6],
                      [0.99, 0.01, 0.0]])]
    NUM_CLASSES = [2, 3]

    def test_top_k_accuracy_evaluator(self):
        gts = [[0.4, 1.0, 1.0], [0.4666666, 0.7333333, 1.0]]
        for k_idx, top_k in enumerate([1, 2]):
            for i, (targets, predictions, num_classes) in enumerate(zip(self.TARGETS, self.PREDICTIONS, self.NUM_CLASSES)):
                metric_topk_acc = MulticlassAccuracy(num_classes=num_classes, top_k=top_k, average='micro')
                metric_topk_acc.update(predictions, targets)
                metric_top_k_acc = metric_topk_acc.compute()
                self.assertAlmostEqual(metric_top_k_acc.item(), gts[i][k_idx], places=5)

    def test_top_1_accuracy_evaluator_equivalent_to_top1_precision_eval(self):
        for i, (targets, predictions, num_classes) in enumerate(zip(self.TARGETS, self.PREDICTIONS, self.NUM_CLASSES)):
            metric_top1_acc = MulticlassAccuracy(num_classes=num_classes, average='micro', top_k=1)
            metric_top1_acc.update(predictions, targets)
            metric_top1_acc = metric_top1_acc.compute()

            metric_top1_prec = MulticlassPrecision(num_classes=num_classes, average='micro', top_k=1)
            metric_top1_prec.update(predictions, targets)
            metric_top1_prec = metric_top1_prec.compute()
            self.assertAlmostEqual(metric_top1_prec, metric_top1_acc, places=5)

    def test_average_precision(self):
        gts = [[0.475744, 0.490476190], [0.485592, 0.50326599]]
        for i, (targets, predictions, num_classes) in enumerate(zip(self.TARGETS, self.PREDICTIONS, self.NUM_CLASSES)):
            for fl_i, flavor in enumerate(['macro', 'weighted']):
                # NOTE: doesnot support samples-based average for multiclass
                metric = MulticlassAveragePrecision(num_classes=num_classes, average=flavor)
                metric.update(predictions, targets)
                metric_avg_prec = metric.compute()
                self.assertAlmostEqual(metric_avg_prec.item(), gts[i][fl_i], places=5)

    def test_ece_loss(self):
        gts = [0.584, 0.40800000]
        for i, (targets, predictions) in enumerate(zip(self.TARGETS, self.PREDICTIONS)):
            metric = MulticlassCalibrationError(num_classes=self.NUM_CLASSES[i])
            metric.update(predictions, targets)
            self.assertAlmostEqual(metric.compute().item(), gts[i], places=5)

    def test_tagwise_accuracy(self):
        metric = MulticlassAccuracy(num_classes=self.NUM_CLASSES[0], average=None)
        metric.update(self.PREDICTIONS[0], self.TARGETS[0])
        metric_tag_wise_acc = metric.compute()
        self.assertAlmostEqual(metric_tag_wise_acc[0].item(), 0.33333, 5)
        self.assertAlmostEqual(metric_tag_wise_acc[1].item(), 0.5, 5)

    def test_perclass_accuracy_with_missing_class(self):
        target_missing_class = torch.tensor([0, 1, 0, 0])
        predicitons_missing_class = torch.tensor([[1, 0, 0],
                                                  [0, 1, 0],
                                                  [0.5, 0.5, 0],
                                                  [0.1, 0.9, 0]])

        metric = MulticlassAccuracy(num_classes=3, average=None)
        metric.update(predicitons_missing_class, target_missing_class)
        metric_tag_wise_acc = metric.compute()
        self.assertAlmostEqual(metric_tag_wise_acc[0].item(), 0.666666, 5)
        self.assertEqual(metric_tag_wise_acc[1].item(), 1.0, 5)
        self.assertEqual(metric_tag_wise_acc[2].item(), 0.0, 5)

    def test_perclass_average_precision(self):
        metric = MulticlassAveragePrecision(num_classes=2, average=None)
        metric.update(self.PREDICTIONS[0], self.TARGETS[0])
        metric_tag_wise_avg_prec = metric.compute()
        self.assertAlmostEqual(metric_tag_wise_avg_prec[0].item(), 0.54940, 5)
        self.assertAlmostEqual(metric_tag_wise_avg_prec[1].item(), 0.40208, 5)


class TestMultilabelClassification(unittest.TestCase):
    TARGETS = torch.tensor([[1, 0, 0],
                            [0, 1, 1],
                            [1, 1, 1]])
    PROB_PREDICTIONS = torch.tensor([[1, 0.31, 0.1],
                                     [0.1, 1, 0.51],
                                     [0.51, 0.61, 0.51]])
    INDEX_PREDICTIONS = torch.tensor([[0, 1, 2],
                                      [1, 2, 0],
                                      [1, 0, 2]])

    def test_multilabel_accuracy(self):
        thresholds = [0.0, 0.3, 0.6, 0.7]
        expectations = [0.66666, 0.88888, 0.66666, 0.55555]
        for i in range(len(thresholds)):
            metric = MultilabelAccuracy(num_labels=3, threshold=thresholds[i])
            metric.update(self.PROB_PREDICTIONS, self.TARGETS)
            metric_acc = metric.compute()
            self.assertAlmostEqual(metric_acc.item(), expectations[i], places=4)

    def test_threshold_based_precision(self):
        thresholds = [0.0, 0.3, 0.6, 0.7]
        expectations = [0.66666, 0.88888, 0.66666, 0.66666]
        for i in range(len(thresholds)):
            metric = MultilabelPrecision(num_labels=3, average='macro', threshold=thresholds[i])
            metric.update(self.PROB_PREDICTIONS, self.TARGETS)
            metric_prec = metric.compute()
            self.assertAlmostEqual(metric_prec.item(), expectations[i], places=4)

    def test_topk_based_precision(self):
        ks = [1, 2, 3]
        expectations = [0.66666, 0.88888, 0.66666]
        for i in range(len(ks)):
            metric = MultilabelPrecision(top_k=ks[i], num_labels=3, average='macro')
            metric.update(self.PROB_PREDICTIONS, self.TARGETS)
            metric = metric.compute()
            self.assertAlmostEqual(metric.item(), expectations[i], places=4)

        # indices mode
        for i in range(len(ks)):
            metric = MultilabelPrecision(top_k=ks[i], prediction_mode='indices', num_labels=3, average='macro')
            metric.update(self.INDEX_PREDICTIONS, self.TARGETS)
            self.assertAlmostEqual(metric.compute().item(), expectations[i], places=4)

    def test_threshold_based_recall(self):
        thresholds = [0.0, 0.3, 0.6, 0.7]
        expectations = [1.0, 1.0, 0.5, 0.3333]
        for i in range(len(thresholds)):
            metric = MultilabelRecall(num_labels=3, average='macro', threshold=thresholds[i])
            metric.update(self.PROB_PREDICTIONS, self.TARGETS)
            metric_prec = metric.compute()
            self.assertAlmostEqual(metric_prec.item(), expectations[i], places=4)

    def test_topk_based_recall(self):
        ks = [0, 1, 2, 3]
        expectations = [0, 0.5, 0.83333, 1.0]
        for i in range(len(ks)):
            metric = MultilabelRecall(top_k=ks[i], num_labels=3, average='macro')
            metric.update(self.PROB_PREDICTIONS, self.TARGETS)
            metric_prec = metric.compute()
            self.assertAlmostEqual(metric_prec.item(), expectations[i], places=4)

        # index based predictions
        for i in range(len(ks)):
            metric = MultilabelRecall(top_k=ks[i], prediction_mode='indices', num_labels=3, average='macro')
            metric.update(self.INDEX_PREDICTIONS, self.TARGETS)
            metric_prec = metric.compute()
            self.assertAlmostEqual(metric_prec.item(), expectations[i], places=4)

    def test_threshold_based_f1_score(self):
        thresholds = [0.0, 0.3, 0.6, 0.7]
        expectations = [0.8, 0.93333, 0.55555, 0.44444]
        for i in range(len(thresholds)):
            metric = MultilabelF1Score(num_labels=3, average='macro', threshold=thresholds[i])
            metric.update(self.PROB_PREDICTIONS, self.TARGETS)
            vmetric_f1 = metric.compute()
            self.assertAlmostEqual(vmetric_f1.item(), expectations[i], places=4)

    def test_topk_based_f1_score(self):
        ks = [0, 1, 2, 3]
        expectations = [0.0, 0.55555, 0.82222, 0.8]
        for i in range(len(ks)):
            metric = MultilabelF1Score(top_k=ks[i], num_labels=3, average='macro')
            metric.update(self.PROB_PREDICTIONS, self.TARGETS)
            metric_f1 = metric.compute()
            self.assertAlmostEqual(metric_f1.item(), expectations[i], places=4)

    def test_average_precision(self):
        targets = torch.tensor([[1, 0, 0, 0],
                                [0, 1, 1, 1],
                                [0, 0, 1, 1],
                                [1, 1, 1, 0]])
        predictions = torch.tensor([[0, 0.3, 0.7, 0],
                                    [0, 1, 0.5, 0],
                                    [0, 0, 0.5, 0],
                                    [0.5, 0.6, 0, 0.5]])
        gts = [0.67328, 0.73611, 0.731481]
        for fl_i, flavor in enumerate(['micro', 'macro', 'weighted']):
            # NOTE: doesnot support samples-based average for multilabel
            vmetric_eval = MultilabelAveragePrecision(num_labels=4, average=flavor)
            vmetric_eval.update(predictions, targets)
            vmetric_avg_prec = vmetric_eval.compute()
            self.assertAlmostEqual(vmetric_avg_prec.item(), gts[fl_i], places=5)


class TestROCAUC(unittest.TestCase):
    @staticmethod
    def _get_metric(predictions, targets, task='multiclass', num_classes=None, average='macro'):
        if task == 'binary':
            metric = BinaryAUROC()
        elif task == 'multiclass':
            metric = MulticlassAUROC(num_classes=num_classes, average=average)
        else:
            metric = MultilabelAUROC(num_labels=num_classes, average=average)

        metric.update(predictions, targets)
        roc_auc = metric.compute()
        return roc_auc

    def test_perfect_predictions(self):
        predictions = torch.tensor([0, 0.1, 0.2, 0.3, 0.4, 0.5])
        targets = torch.tensor([0, 0, 0, 1, 1, 1])
        roc_auc = self._get_metric(predictions, targets, task='binary', num_classes=2)
        self.assertAlmostEqual(roc_auc.item(), 1.0, places=4)

    def test_abysmal_predictions(self):
        predictions = torch.tensor([0, 0.1, 0.2, 0.3, 0.4, 0.5])
        targets = torch.tensor([1, 1, 1, 0, 0, 0])
        roc_auc = self._get_metric(predictions, targets, task='binary', num_classes=2)
        self.assertAlmostEqual(roc_auc.item(), 0.0, places=4)

    def test_imperfect_predictions(self):
        predictions = torch.tensor([0, 0.1, 0.2, 0.3, 0.4, 0.5])
        targets = torch.tensor([0, 0, 0, 1, 0, 1])
        roc_auc = self._get_metric(predictions, targets, task='binary', num_classes=2)
        self.assertAlmostEqual(roc_auc.item(), 0.875, places=4)

    def test_multiclass_perfect_predictions(self):
        predictions = torch.tensor([[0.8, 0.2, 0.0], [0.7, 0.2, 0.1], [0.1, 0.6, 0.3], [0.2, 0.7, 0.1], [0.1, 0.3, 0.6], [0.1, 0.3, 0.6]])
        targets = torch.tensor([0, 0, 1, 1, 2, 2])
        roc_auc = self._get_metric(predictions, targets, task='multiclass', num_classes=3)
        self.assertAlmostEqual(roc_auc.item(), 1.0, places=4)

    def test_multilabel_perfect_predictions(self):
        predictions = torch.tensor([[0.8, 0.2, 0.0], [0.7, 0.2, 0.1], [0.1, 0.6, 0.3], [0.2, 0.7, 0.1], [0.1, 0.3, 0.6], [0.1, 0.3, 0.6]])
        targets = torch.tensor([[1, 0, 0], [1, 0, 0], [0, 1, 1], [0, 1, 0], [0, 1, 1], [0, 1, 1]])
        roc_auc = self._get_metric(predictions, targets, task='multilabel', num_classes=3)
        self.assertAlmostEqual(roc_auc.item(), 1.0, places=4)


if __name__ == "__main__":
    unittest.main()



from visionmetrics.caption.caption_eval_base import ImageCaptionEvaluatorBase


class CIDErScore(ImageCaptionEvaluatorBase):
    """
    CIDEr score evaluator for image caption task. For more details, refer to http://arxiv.org/pdf/1411.5726.pdf.
    """

    def __init__(self):
        super().__init__(metric='CIDEr')



import cv2
import torch
from torchmetrics import Metric


class MattingEvaluatorBase(Metric):
    """
    Base class for image matting evaluator
    """

    def __init__(self, metric):
        super().__init__()
        self._metric = metric
        self.add_state("_num_samples", default=torch.tensor(0, dtype=torch.float32), dist_reduce_fx="sum")
        self.add_state("_metric_sum", default=torch.tensor(0, dtype=torch.float32), dist_reduce_fx="sum")

    def _convert2binary(self, mask, threshold=128):
        bin_mask = mask >= threshold
        return bin_mask

    def _preprocess(self, pred_mask: torch.tensor, gt_mask: torch.tensor):

        pred_binmask = self._convert2binary(pred_mask)
        gt_binmask = self._convert2binary(gt_mask)
        return pred_binmask, gt_binmask

    def _find_contours(self, matting, thickness=10):
        matting = matting.clone().detach()
        opencv_major_version = int(cv2.__version__.split('.')[0])
        if opencv_major_version >= 4:
            contours, _ = cv2.findContours(matting.numpy(), cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)
        else:
            _, contours, _ = cv2.findContours(matting.numpy(), cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)
        mask = torch.zeros(matting.shape, dtype=torch.uint8)

        cv2.drawContours(mask.numpy(), contours, -1, 255, thickness)
        return mask

    def _create_contour_mask(self, gt_mask, pred_mask, line_width=10):
        contour_mask = self._find_contours((gt_mask * 255).to(torch.uint8), thickness=line_width) / 255.0
        gt_contour_mask = gt_mask * contour_mask
        pred_contour_mask = pred_mask * contour_mask
        return gt_contour_mask, pred_contour_mask

    def compute(self):
        return {self._metric: self._metric_sum / self._num_samples if self._num_samples else 0.0}



from .caption_eval_base import ImageCaptionEvaluatorBase


class METEORScore(ImageCaptionEvaluatorBase):
    """
    METEOR score evaluator for image caption task. For more details, refer to http://www.cs.cmu.edu/~alavie/METEOR/.
    """

    def __init__(self):
        super().__init__(metric='METEOR')



import unittest
import torch
from visionmetrics.prediction_filters import TopKPredictionFilter


class TestTopKPredictionFilter(unittest.TestCase):

    PROB_PREDS = torch.tensor([[1, 0.3, 0],
                               [0, 1, 0.5],
                               [0.5, 0.6, 0.5]])

    INDEX_PREDS = torch.tensor([[0, 1, 2],
                                [1, 2, 0],
                                [1, 0, 2]])

    EXPECTED_TOPK_PREDS = [torch.tensor([[0, 0, 0],
                                         [0, 0, 0],
                                         [0, 0, 0]]),
                           torch.tensor([[1, 0, 0],
                                         [0, 1, 0],
                                         [0, 1, 0]]),
                           torch.tensor([[1, 1, 0],
                                         [0, 1, 1],
                                         [1, 1, 0]]),
                           torch.tensor([[1, 1, 1],
                                         [1, 1, 1],
                                         [1, 1, 1]])]
    K = [0, 1, 2, 3]

    def test_topk_prediction_filter_prob_mode(self):
        for i, k in enumerate(self.K):
            pred_filter = TopKPredictionFilter(k=k, prediction_mode='prob')
            topk_preds = pred_filter.filter(self.PROB_PREDS, return_mode='vec')
            self.assertTrue(torch.equal(topk_preds, self.EXPECTED_TOPK_PREDS[i]))

    def test_topk_prediction_filter_indices_mode(self):
        for i, k in enumerate(self.K):
            pred_filter = TopKPredictionFilter(k=k, prediction_mode='indices')
            topk_preds = pred_filter.filter(self.INDEX_PREDS, return_mode='vec')
            self.assertTrue(torch.equal(topk_preds, self.EXPECTED_TOPK_PREDS[i]))


if __name__ == '__main__':
    unittest.main()



from .foreground_iou import ForegroundIOU
from .boundary_foreground_iou import BoundaryForegroundIOU
from .mean_iou import MeanIOU
from .l1_error import L1Error
from .boundary_mean_iou import BoundaryMeanIOU

__all__ = ["ForegroundIOU", "BoundaryForegroundIOU", "MeanIOU", "L1Error", "BoundaryMeanIOU"]
